//
//  DetailViewController.m
//  AnalogSynthesizerPad
//
//  Created by 成田 拓郎 on 10/07/19.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "DetailViewController.h"
#import "RootViewController.h"


@interface DetailViewController ()
@property (nonatomic, retain) UIPopoverController *popoverController;
@end



@implementation DetailViewController

@synthesize toolbar, popoverController;
@synthesize keyboardView;

@synthesize vcoWaveTypeVolumeView;
@synthesize vcoPulseWidthVolumeView;
@synthesize vcoControlLevelVolumeView;

@synthesize vcfCutOffFrequencyVolumeView;
@synthesize vcfResonanceVolumeView;
@synthesize vcfControlLevelVolumeView;

@synthesize vcaControlLevelVolumeView;

@synthesize egParamAttackVolumeView;
@synthesize egParamDecayVolumeView;
@synthesize egParamSustainVolumeView;
@synthesize egParamReleaseVolumeView;

@synthesize lfoWaveTypeVolumeView;
@synthesize lfoPulseWidthVolumeView;
@synthesize lfoFrequencyVolumeView;

@synthesize echoDelayTimeVolumeView;
@synthesize echoDelayLevelVolumeView;

@synthesize synthesizerOutput;

#pragma mark -
#pragma mark Managing the detail item

/*
 RootViewが選択された場合の処理
 */
- (void)selectRootView:(int)row {
	float eg_a, eg_d, eg_s, eg_r;
	switch (row) {
		case 0:	//Organ
		default:
			eg_a = 0.05f; eg_d = 0.0f; eg_s = 1.0f; eg_r = 0.05f;
			break;
		case 1:	//Piano
			eg_a = 0.1f; eg_d = 0.7f; eg_s = 0.0f; eg_r = 0.1f;
			break;
		case 2:	//Lead Guitar
			eg_a = 0.1f; eg_d = 0.3f; eg_s = 0.5f; eg_r = 0.5f;
			break;
		case 3:	//Strings
			eg_a = 1.0f; eg_d = 0.5f; eg_s = 0.8f; eg_r = 0.5f;
			break;
	}

	self.synthesizerOutput.egParamAttack = eg_a / 2.0f;
	egParamAttackVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamAttack * 2.0f * 127.0f);
	[egParamAttackVolumeView rotateView];
	
	self.synthesizerOutput.egParamDecay = eg_d;
	egParamDecayVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamDecay * 127.0f);
	[egParamDecayVolumeView rotateView];
	
	self.synthesizerOutput.egParamSustain = eg_s;
	egParamSustainVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamSustain * 127.0f);
	[egParamSustainVolumeView rotateView];
	
	self.synthesizerOutput.egParamRelease = eg_r;
	egParamReleaseVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamRelease * 127.0f);
	[egParamReleaseVolumeView rotateView];

    if (popoverController != nil) {
        [popoverController dismissPopoverAnimated:YES];
    }        
}

#pragma mark -
#pragma mark Split view support

- (void)splitViewController: (UISplitViewController*)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem*)barButtonItem forPopoverController: (UIPopoverController*)pc {
    
    barButtonItem.title = @"Type List";
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items insertObject:barButtonItem atIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = pc;
}


// Called when the view is shown again in the split view, invalidating the button and popover controller.
- (void)splitViewController: (UISplitViewController*)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem {
    
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items removeObjectAtIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = nil;
}


#pragma mark -
#pragma mark Rotation support

// Ensure that the view controller supports rotation and that the split view can therefore show in both portrait and landscape.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
										 duration:(NSTimeInterval)duration {
	if (toInterfaceOrientation == UIInterfaceOrientationPortrait ||
		toInterfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		// ポートレート モード
	}
	else {
		// ランドスケープ モード
	}
	vcoWaveTypeVolumeView.center     = volumeCenter[0];
	vcoPulseWidthVolumeView.center   = volumeCenter[1];
	vcoControlLevelVolumeView.center = volumeCenter[2];
	vcfCutOffFrequencyVolumeView.center = volumeCenter[3];
	vcfResonanceVolumeView.center       = volumeCenter[4];
	vcfControlLevelVolumeView.center    = volumeCenter[5];
	vcaControlLevelVolumeView.center = volumeCenter[6];
	egParamAttackVolumeView.center  = volumeCenter[7];
	egParamDecayVolumeView.center   = volumeCenter[8];
	egParamSustainVolumeView.center = volumeCenter[9];
	egParamReleaseVolumeView.center = volumeCenter[10];
	lfoWaveTypeVolumeView.center   = volumeCenter[11];
	lfoPulseWidthVolumeView.center = volumeCenter[12];
	lfoFrequencyVolumeView.center  = volumeCenter[13];
	echoDelayTimeVolumeView.center  = volumeCenter[14];
	echoDelayLevelVolumeView.center = volumeCenter[15];

	CGRect rect = CGRectMake(0, 0, 96, 96);
	vcoWaveTypeVolumeView.bounds     = rect;
	vcoPulseWidthVolumeView.bounds   = rect;
	vcoControlLevelVolumeView.bounds = rect;
	vcfCutOffFrequencyVolumeView.bounds = rect;
	vcfResonanceVolumeView.bounds       = rect;
	vcfControlLevelVolumeView.bounds    = rect;
	vcaControlLevelVolumeView.bounds = rect;
	egParamAttackVolumeView.bounds  = rect;
	egParamDecayVolumeView.bounds   = rect;
	egParamSustainVolumeView.bounds = rect;
	egParamReleaseVolumeView.bounds = rect;
	lfoWaveTypeVolumeView.bounds   = rect;
	lfoPulseWidthVolumeView.bounds = rect;
	lfoFrequencyVolumeView.bounds  = rect;
	echoDelayTimeVolumeView.bounds  = rect;
	echoDelayLevelVolumeView.bounds = rect;
}

#pragma mark -
#pragma mark View lifecycle

//オーディオセッション　割り込みリスナーコールバック関数
static void interruptionListenerCallback(void *inUserData, UInt32 interruptionState)
{
	DetailViewController *controller = (DetailViewController *)inUserData;
	
	if (interruptionState == kAudioSessionBeginInterruption)
	{
		//オーディオセッション割り込みの開始
		//再生の停止
		controller.synthesizerOutput.play = NO;
	}
	else if (interruptionState == kAudioSessionEndInterruption)
	{
		//オーディオセッション割り込みの終了
		//オーディオセッションのカテゴリを設定
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
								sizeof(sessionCategory),
								&sessionCategory);
		//オーディオセッションのアクティブ化
		AudioSessionSetActive(true);
		//再生の再開
		controller.synthesizerOutput.play = YES;
	}
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

	//オーディオセッションを初期化
	AudioSessionInitialize(NULL, NULL, interruptionListenerCallback, self);
	//オーディオセッションのカテゴリを設定
	UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
	AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
							sizeof(sessionCategory),
							&sessionCategory);
	//バッファサイズを設定
	Float32 bufferSize = 0.02f; // 0.02秒
	AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareIOBufferDuration,
							sizeof(bufferSize),
							&bufferSize);
	//オーディオセッションのアクティブ化
	AudioSessionSetActive(true);
	
	//AudioUnitの準備
	SynthesizerOutput *synthe = [[SynthesizerOutput alloc] init];
	self.synthesizerOutput = synthe;
	[synthe release];

	// 再生開始
	self.synthesizerOutput.play = YES;

	//UIコントロールの準備
	[keyboardView setAction:@selector(keyboardChanged:)];
	[keyboardView setTarget:self];
	
	[vcoWaveTypeVolumeView setAction:@selector(vcoWaveTypeChanged:)];
	[vcoWaveTypeVolumeView setTarget:self];
	vcoWaveTypeVolumeView.valueKnob = self.synthesizerOutput.vcoWaveType;
	vcoWaveTypeVolumeView.valueMax  = 5;
	[vcoWaveTypeVolumeView rotateView];
	volumeCenter[0] = vcoWaveTypeVolumeView.center;
	
	[vcoPulseWidthVolumeView setAction:@selector(vcoPulseWidthChanged:)];
	[vcoPulseWidthVolumeView setTarget:self];
	vcoPulseWidthVolumeView.valueKnob = (int)(self.synthesizerOutput.vcoPulseWidth * 127.0f);
	vcoPulseWidthVolumeView.valueMax = 127;
	[vcoPulseWidthVolumeView rotateView];
	volumeCenter[1] = vcoPulseWidthVolumeView.center;
	
	[vcoControlLevelVolumeView setAction:@selector(vcoControlLevelChanged:)];
	[vcoControlLevelVolumeView setTarget:self];
	vcoControlLevelVolumeView.valueKnob = (int)(self.synthesizerOutput.vcoControlLevel * 127.0f);
	vcoControlLevelVolumeView.valueMax = 127;
	[vcoControlLevelVolumeView rotateView];
	volumeCenter[2] = vcoControlLevelVolumeView.center;
	
	[vcfCutOffFrequencyVolumeView setAction:@selector(vcfCutOffFrequencyChanged:)];
	[vcfCutOffFrequencyVolumeView setTarget:self];
	vcfCutOffFrequencyVolumeView.valueKnob = (int)((self.synthesizerOutput.vcfCutOffFrequency - 100.0f) * 127.0f / 9900.0f);
	vcfCutOffFrequencyVolumeView.valueMax = 127;
	[vcfCutOffFrequencyVolumeView rotateView];
	volumeCenter[3] = vcfCutOffFrequencyVolumeView.center;
	
	[vcfResonanceVolumeView setAction:@selector(vcfResonanceChanged:)];
	[vcfResonanceVolumeView setTarget:self];
	vcfResonanceVolumeView.valueKnob = (int)(self.synthesizerOutput.vcfResonance * 127.0f / 2.0f);
	vcfResonanceVolumeView.valueMax = 127;
	[vcfResonanceVolumeView rotateView];
	volumeCenter[4] = vcfResonanceVolumeView.center;
	
	[vcfControlLevelVolumeView setAction:@selector(vcfControlLevelChanged:)];
	[vcfControlLevelVolumeView setTarget:self];
	vcfControlLevelVolumeView.valueKnob = (int)(self.synthesizerOutput.vcfControlLevel * 127.0f);
	vcfControlLevelVolumeView.valueMax = 127;
	[vcfControlLevelVolumeView rotateView];
	volumeCenter[5] = vcfControlLevelVolumeView.center;
	
	[vcaControlLevelVolumeView setAction:@selector(vcaControlLevelChanged:)];
	[vcaControlLevelVolumeView setTarget:self];
	vcaControlLevelVolumeView.valueKnob = (int)(self.synthesizerOutput.vcaControlLevel * 127.0f);
	vcaControlLevelVolumeView.valueMax = 127;
	[vcaControlLevelVolumeView rotateView];
	volumeCenter[6] = vcaControlLevelVolumeView.center;
	
	[egParamAttackVolumeView setAction:@selector(egParamAttackChanged:)];
	[egParamAttackVolumeView setTarget:self];
	egParamAttackVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamAttack * 2.0f * 127.0f);
	egParamAttackVolumeView.valueMax = 127;
	[egParamAttackVolumeView rotateView];
	volumeCenter[7] = egParamAttackVolumeView.center;
	
	[egParamDecayVolumeView setAction:@selector(egParamDecayChanged:)];
	[egParamDecayVolumeView setTarget:self];
	egParamDecayVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamDecay * 127.0f);
	egParamDecayVolumeView.valueMax = 127;
	[egParamDecayVolumeView rotateView];
	volumeCenter[8] = egParamDecayVolumeView.center;
	
	[egParamSustainVolumeView setAction:@selector(egParamSustainChanged:)];
	[egParamSustainVolumeView setTarget:self];
	egParamSustainVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamSustain * 127.0f);
	egParamSustainVolumeView.valueMax = 127;
	[egParamSustainVolumeView rotateView];
	volumeCenter[9] = egParamSustainVolumeView.center;
	
	[egParamReleaseVolumeView setAction:@selector(egParamReleaseChanged:)];
	[egParamReleaseVolumeView setTarget:self];
	egParamReleaseVolumeView.valueKnob = (int)(self.synthesizerOutput.egParamRelease * 127.0f);
	egParamReleaseVolumeView.valueMax = 127;
	[egParamReleaseVolumeView rotateView];
	volumeCenter[10] = egParamReleaseVolumeView.center;
	
	[lfoWaveTypeVolumeView setAction:@selector(lfoWaveTypeChanged:)];
	[lfoWaveTypeVolumeView setTarget:self];
	lfoWaveTypeVolumeView.valueKnob = self.synthesizerOutput.lfoWaveType;
	lfoWaveTypeVolumeView.valueMax = 5;
	[lfoWaveTypeVolumeView rotateView];
	volumeCenter[11] = lfoWaveTypeVolumeView.center;
	
	[lfoPulseWidthVolumeView setAction:@selector(lfoPulseWidthChanged:)];
	[lfoPulseWidthVolumeView setTarget:self];
	lfoPulseWidthVolumeView.valueKnob = (int)(self.synthesizerOutput.lfoPulseWidth * 127.0f);
	lfoPulseWidthVolumeView.valueMax = 127;
	[lfoPulseWidthVolumeView rotateView];
	volumeCenter[12] = lfoPulseWidthVolumeView.center;
	
	[lfoFrequencyVolumeView setAction:@selector(lfoFrequencyChanged:)];
	[lfoFrequencyVolumeView setTarget:self];
	lfoFrequencyVolumeView.valueKnob = (int)((self.synthesizerOutput.lfoFrequency - 5.0f) * 127.0f / 95.0f);
	lfoFrequencyVolumeView.valueMax = 127;
	[lfoFrequencyVolumeView rotateView];
	volumeCenter[13] = lfoFrequencyVolumeView.center;
	
	[echoDelayTimeVolumeView setAction:@selector(echoDelayTimeChanged:)];
	[echoDelayTimeVolumeView setTarget:self];
	echoDelayTimeVolumeView.valueKnob = (int)(0.1f * 127.0f);
	echoDelayTimeVolumeView.valueMax = 127;
	[echoDelayTimeVolumeView rotateView];
	volumeCenter[14] = echoDelayTimeVolumeView.center;
	
	[echoDelayLevelVolumeView setAction:@selector(echoDelayLevelChanged:)];
	[echoDelayLevelVolumeView setTarget:self];
	echoDelayLevelVolumeView.valueKnob = 0;
	echoDelayLevelVolumeView.valueMax = 127;
	[echoDelayLevelVolumeView rotateView];
	volumeCenter[15] = echoDelayLevelVolumeView.center;
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/

- (void)viewDidUnload {
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    self.popoverController = nil;

	self.keyboardView = nil;
	
	self.vcoWaveTypeVolumeView = nil;
	self.vcoPulseWidthVolumeView = nil;
	self.vcoControlLevelVolumeView = nil;
	
	self.vcfCutOffFrequencyVolumeView = nil;
	self.vcfResonanceVolumeView = nil;
	self.vcfControlLevelVolumeView = nil;
	
	self.vcaControlLevelVolumeView = nil;
	
	self.egParamAttackVolumeView = nil;
	self.egParamDecayVolumeView = nil;
	self.egParamSustainVolumeView = nil;
	self.egParamReleaseVolumeView = nil;
	
	self.lfoWaveTypeVolumeView = nil;
	self.lfoPulseWidthVolumeView = nil;
	self.lfoFrequencyVolumeView = nil;
}


#pragma mark -
#pragma mark Memory management

/*
- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}
*/

- (void)dealloc {
    [popoverController release];
    [toolbar release];

	[keyboardView release];
	
	[vcoWaveTypeVolumeView release];
	[vcoPulseWidthVolumeView release];
	[vcoControlLevelVolumeView release];
	
	[vcfCutOffFrequencyVolumeView release];
	[vcfResonanceVolumeView release];
	[vcfControlLevelVolumeView release];
	
	[vcaControlLevelVolumeView release];
	
	[egParamAttackVolumeView release];
	[egParamDecayVolumeView release];
	[egParamSustainVolumeView release];
	[egParamReleaseVolumeView release];
	
	[lfoWaveTypeVolumeView release];
	[lfoPulseWidthVolumeView release];
	[lfoFrequencyVolumeView release];
	
	[synthesizerOutput release];

    [super dealloc];
}

- (void)keyboardChanged:(id)sender {
//	NSLog(@"Keyboard Message");
	synthesizerOutput.keyboard = keyboardView.keyDown;
}

-(void)vcoWaveTypeChanged:(id)sender {
	synthesizerOutput.vcoWaveType = vcoWaveTypeVolumeView.valueKnob;
}
-(void)vcoPulseWidthChanged:(id)sender {
	float v = vcoPulseWidthVolumeView.valueKnob;
	float vm = vcoPulseWidthVolumeView.valueMax;
	synthesizerOutput.vcoPulseWidth = v / vm;
}
-(void)vcoControlLevelChanged:(id)sender {
	float v = vcoControlLevelVolumeView.valueKnob;
	float vm = vcoControlLevelVolumeView.valueMax;
	synthesizerOutput.vcoControlLevel = v / vm;
}

-(void)vcfCutOffFrequencyChanged:(id)sender {
	float v = vcfCutOffFrequencyVolumeView.valueKnob;
	float vm = vcfCutOffFrequencyVolumeView.valueMax;
	synthesizerOutput.vcfCutOffFrequency = 100.0f + (9900.0f * v / vm);
}
-(void)vcfResonanceChanged:(id)sender {
	float v = vcfResonanceVolumeView.valueKnob;
	float vm = vcfResonanceVolumeView.valueMax;
	synthesizerOutput.vcfResonance = 0.25f + 1.75f * v / vm;
}
-(void)vcfControlLevelChanged:(id)sender {
	float v = vcfControlLevelVolumeView.valueKnob;
	float vm = vcfControlLevelVolumeView.valueMax;
	synthesizerOutput.vcfControlLevel = v / vm;
}

-(void)vcaControlLevelChanged:(id)sender {
	float v = vcaControlLevelVolumeView.valueKnob;
	float vm = vcaControlLevelVolumeView.valueMax;
	synthesizerOutput.vcaControlLevel = v / vm;
}

-(void)egParamAttackChanged:(id)sender {
	float v = egParamAttackVolumeView.valueKnob;
	float vm = egParamAttackVolumeView.valueMax;
	synthesizerOutput.egParamAttack = 0.5f * v / vm;
}
-(void)egParamDecayChanged:(id)sender {
	float v = egParamDecayVolumeView.valueKnob;
	float vm = egParamDecayVolumeView.valueMax;
	synthesizerOutput.egParamDecay = v / vm;
}
-(void)egParamSustainChanged:(id)sender {
	float v = egParamSustainVolumeView.valueKnob;
	float vm = egParamSustainVolumeView.valueMax;
	synthesizerOutput.egParamSustain = v / vm;
}
-(void)egParamReleaseChanged:(id)sender {
	float v = egParamReleaseVolumeView.valueKnob;
	float vm = egParamReleaseVolumeView.valueMax;
	synthesizerOutput.egParamRelease = v / vm;
}

-(void)lfoWaveTypeChanged:(id)sender {
	synthesizerOutput.lfoWaveType = lfoWaveTypeVolumeView.valueKnob;
}
-(void)lfoPulseWidthChanged:(id)sender {
	float v = lfoPulseWidthVolumeView.valueKnob;
	float vm = lfoPulseWidthVolumeView.valueMax;
	synthesizerOutput.lfoPulseWidth = v / vm;
}
-(void)lfoFrequencyChanged:(id)sender {
	float v = lfoFrequencyVolumeView.valueKnob;
	float vm = lfoFrequencyVolumeView.valueMax;
	synthesizerOutput.lfoFrequency = 5.0f + (95.0f * v / vm);
}

-(void)echoDelayTimeChanged:(id)sender
{
	float v = echoDelayTimeVolumeView.valueKnob;
	float vm = echoDelayTimeVolumeView.valueMax;
	synthesizerOutput.echoDelayTime = v / vm;
}
-(void)echoDelayLevelChanged:(id)sender
{
	float v = echoDelayLevelVolumeView.valueKnob;
	float vm = echoDelayLevelVolumeView.valueMax;
	synthesizerOutput.echoDelayLevel = v / vm;
}

@end
